#!/usr/bin/env python3
"""
SAFLA Setup Script

This setup.py file is maintained for backward compatibility.
The primary configuration is in pyproject.toml.
"""

from setuptools import setup

if __name__ == "__main__":
    setup()